// Copyright 2020 Andy Grove
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.ballistacompute.physical.expressions

import org.ballistacompute.datatypes.ArrowTypes
import org.ballistacompute.datatypes.ColumnVector
import org.ballistacompute.datatypes.LiteralValueVector
import org.ballistacompute.datatypes.RecordBatch

/** Physical representation of an expression. */
interface Expression {

  /**
   * Evaluate the expression against an input record batch and produce a column of data as output
   */
  fun evaluate(input: RecordBatch): ColumnVector
}

class LiteralLongExpression(val value: Long) : Expression {
  override fun evaluate(input: RecordBatch): ColumnVector {
    return LiteralValueVector(ArrowTypes.Int64Type, value, input.rowCount())
  }
}

class LiteralDoubleExpression(val value: Double) : Expression {
  override fun evaluate(input: RecordBatch): ColumnVector {
    return LiteralValueVector(ArrowTypes.DoubleType, value, input.rowCount())
  }
}

class LiteralStringExpression(val value: String) : Expression {
  override fun evaluate(input: RecordBatch): ColumnVector {
    return LiteralValueVector(ArrowTypes.StringType, value.toByteArray(), input.rowCount())
  }
}

interface Accumulator {
  fun accumulate(value: Any?)
  fun finalValue(): Any?
}
